---
title: "Simple Detector"
---

## Setup 

Note all these commands assume you are in an `eic-shell` singularity session.

```bash
git clone https://eicweb.phy.anl.gov/EIC/tutorials/ip6_tutorial_1.git part1
cd part1
```

## Inspect files 

Look around see what's what.
The files 
[`compact/materials.xml`](https://eicweb.phy.anl.gov/EIC/tutorials/ip6_tutorial_1/blob/master/compact/materials.xml), 
[`compact/elements.xml`](https://eicweb.phy.anl.gov/EIC/tutorials/ip6_tutorial_1/blob/master/compact/elements.xml),
and 
[`gem_tracker.xml`](https://eicweb.phy.anl.gov/EIC/tutorials/ip6_tutorial_1/blob/master/gem_tracker.xml) 
are the compact detector description files. The first two files are included in 
the main `gem_tracker.xml` file. Note these files are often found in 
directories named compact (but not always).

## Compile the detector library

```bash
mkdir build
cmake ../. -DCMAKE_INSTALL_PREFIX=../../local 
make -j4 install
```

## Look at geometry


```bash
dd_web_display --export gem_tracker.xml
```
Copy the generated file `detector_geometry.root` to your local machine.
Then open the [web viewer](https://eic.phy.anl.gov/geoviewer/), click on the `...` and open the local root file.

![simple gem tracker](../gem_tracker_part1.png)

Examine the compact description, specifically this part:

```xml
<detector id="2" name="GEMTracker" vis="RedVis" type="my_GEMTracker" readout="GEMTrackerHits" > 
  <layer id="1" z="-100 *cm" inner_r="40*cm"  outer_r="120*cm"   phi0_offset="0.0*deg" />
  <layer id="2" z="-80 *cm"  inner_r="30*cm"  outer_r="90*cm"    phi0_offset="0.0*deg" />
  <layer id="3" z="-60 *cm"  inner_r="20*cm"  outer_r="70*cm"    phi0_offset="0.0*deg" />
  <layer id="4" z="-40 *cm"  inner_r="20*cm"  outer_r="50.0*cm"  phi0_offset="0.0*deg" />
  <layer id="5" z=" 40 *cm"  inner_r="20*cm"  outer_r="50.0*cm" phi0_offset="0.0*deg" />
  <layer id="6" z=" 60 *cm"  inner_r="25*cm"  outer_r="70.0*cm" phi0_offset="0.0*deg" />
  <layer id="7" z=" 80 *cm"  inner_r="30*cm"  outer_r="90.0*cm" phi0_offset="0.0*deg" />
  <layer id="8" z="100 *cm"  inner_r="40*cm"  outer_r="100.0*cm" phi0_offset="0.0*deg" />
</detector>
```

## Run the geant4 simulation

```bash
npsim  --runType run  --enableG4GPS \
   --macroFile gps.mac \
   --compactFile ./gem_tracker.xml \
   --outputFile gem_tracker_sim.root 
```

This will run geant4 using the General Particle Source (GPS) tool.
The output will be a root file containing the generated events and GEM tracker hits.

See the [GPS Documentation](http://www.fe.infn.it/u/paterno/Geant4_tutorial/slides_further/GPS/GPS.pdf) for more information.

## Looking at the data 

First, the data model used by npsim is described here
https://eic.phy.anl.gov/eicd/

## Hit position

```bash
root -b -q scripts/tutorial2_cell_size.cxx+
```
Look at the generated file in results:
![local hit position](../hit_position.png)

This is the local position in the segmentation (i.e. pixel, strip, readout pad, etc.).

### Cell ID (channel) to Position


At the top of the script you can see these lines:
```cpp
  dd4hep::Detector& detector = dd4hep::Detector::getInstance();
  detector.fromCompact("gem_tracker.xml");
  dd4hep::rec::CellIDPositionConverter cellid_converter(detector);
```
This 
  - gets the DD4hep instance
  - loads the compact detector file
  - initializes the [position converter tool](https://dd4hep.web.cern.ch/dd4hep/reference/classdd4hep_1_1rec_1_1CellIDPositionConverter.html) (which provides thread safe access)

Later on you can see its use:
```cpp
  auto pos1 = cellid_converter.position(h.cellID);
```

### Cell size

You will see this:
```bash
...
Segmentation-Cell Position : 39,102,-100
 dim 1, 3, 
```


### Id Specification

```bash
root -b -q scripts/tutorial3_id_spec.cxx+
```

```cpp
  fmt::print("--------------------------\n");
  fmt::print("ID specification:\n");
  auto decoder = detector.readout("GEMTrackerHits").idSpec().decoder();
  fmt::print("{}\n", decoder->fieldDescription());
  auto layer_index = decoder->index("layer");
  fmt::print(" \"layer\" index is {}.\n", layer_index);
```

Later in the loop over event hits.
```cpp
      auto detector_layer = decoder->get(h.cellID, layer_index);
      if ((detector_layer !=4 ) && (detector_layer !=5 )) {
        continue;
      }
```
Note the `layer_index` is computed once in the beginning when the decoder is also initialized. 
This means it will be fast because it only parses the ID specification string once.

```bash
ID specification:
system:0:5,barrel:5:3,layer:8:4,module:12:5,r:32:-16,phi:48:-16
 "layer" index is 2.
```
or
```bash
ID specification:
system:0:8,barrel:8:2,layer:10:4,module:14:12,sensor:26:2,x:32:-16,y:48:-16
 "layer" index is 2.
```

For more information, see the [BitFieldCoder documentation](https://dd4hep.web.cern.ch/dd4hep/reference/classdd4hep_1_1DDSegmentation_1_1BitFieldCoder.html).


